@@ -1,9 +1,8 @@
-#!/usr/bin/env python
-# -*- coding: UTF-8 -*-
+from __future__ import annotations as a
+
+import dataclasses
 import functools
 import inspect
-import dataclasses
-import os
 from operator import attrgetter
 from types import FunctionType, MethodType
 from typing import (
@@ -23,8 +22,9 @@ from typing import (
 )
 
 import typic.constraints as c
-from typic.checks import issubclass, ishashable, isfrozendataclass
+from typic.checks import issubclass, isfrozendataclass
 from typic.compat import lru_cache
+from typic.env import Environ, EnvironmentTypeError, EnvironmentValueError
 from typic.serde.binder import BoundArguments
 from typic.serde.common import (
     Annotation,
@@ -56,7 +56,7 @@ from typic.strict import (
     StrictModeT,
 )
 from typic.ext.schema import SchemaFieldT, builder as schema_builder, ObjectSchemaField
-from typic.util import origin
+from typic.util import origin, cached_type_hints
 from typic.types import FrozenDict, freeze
 
 __all__ = (
@@ -67,6 +67,12 @@ __all__ = (
     "Case",
     "coerce",
     "constrained",
+    "decode",
+    "encode",
+    "environ",
+    "EnvironmentTypeError",
+    "EnvironmentValueError",
+    "flags",
     "is_strict_mode",
     "iterate",
     "tojson",
@@ -112,6 +118,9 @@ protocols = resolver.protocols
 protocol = resolver.resolve
 tojson = resolver.tojson
 iterate = resolver.iterate
+flags = SerdeFlags
+encode = resolver.encode
+decode = resolver.decode
 
 # TBDeprecated
 coerce = resolver.coerce_value
@@ -187,6 +196,8 @@ def _bind_proto(cls, proto: SerdeProtocol):
         ("transmute", staticmethod(proto.transmute)),
         ("validate", staticmethod(proto.validate)),
         ("translate", proto.translate),
+        ("encode", proto.encode),
+        ("decode", staticmethod(proto.decode)),
     ):
         setattr(cls, n, attr)
 
@@ -196,6 +207,7 @@ def _resolve_class(
     cls: Type[ObjectT],
     *,
     strict: StrictModeT = STRICT_MODE,
+    always: bool = True,
     jsonschema: bool = True,
     serde: SerdeFlags = None,
 ) -> Type[WrappedObjectT]:
@@ -229,7 +241,7 @@ def _resolve_class(
                 __setter(
                     self,
                     name,
-                    __trans[name](item) if name in protos else item,
+                    __trans[name](item) if name in __trans else item,
                 )
 
             return __setattr_typed__
@@ -492,7 +504,7 @@ def constrained(
     >>> ShortStr('waytoomanycharacters')
     Traceback (most recent call last):
     ...
-    typic.constraints.error.ConstraintValueError: Given value <'waytoomanycharacters'> fails constraints: (type=str, nullable=False, coerce=False, max_length=10)
+    typic.constraints.error.ConstraintValueError: Given value <'waytoomanycharacters'> fails constraints: (type=str, nullable=False, max_length=10)
     >>> @typic.constrained(values=ShortStr, max_items=2)
     ... class SmallMap(dict):
     ...     '''A small map that only allows short strings.'''
@@ -579,39 +591,7 @@ def constrained(
     return constr_wrapper(_klass) if _klass else constr_wrapper
 
 
-def _resolve_from_env(
-    cls: Type[ObjectT],
-    prefix: str,
-    case_sensitive: bool,
-    aliases: Mapping[str, str],
-    *,
-    environ: Mapping[str, str] = None,
-) -> Type[ObjectT]:
-    environ = environ or os.environ
-    env = {(x.lower() if not case_sensitive else x): y for x, y in environ.items()}
-    fields = {
-        (f"{prefix}{x}".lower() if not case_sensitive else f"{prefix}{x}"): (x, y)
-        for x, y in cls.__annotations__.items()
-    }
-    names = {*fields, *aliases}
-    sentinel = object()
-    for k in env.keys() & names:
-        name = aliases.get(k, k)
-        attr, typ = fields[name]
-        val = transmute(typ, env[k])
-        use_factory = not ishashable(val)
-        field = getattr(cls, attr, sentinel)
-        if not isinstance(field, dataclasses.Field):
-            field = dataclasses.field()
-        if use_factory:
-            field.default_factory = lambda x=val: x
-            field.default = dataclasses.MISSING
-        else:
-            field.default = val
-            field.default_factory = dataclasses.MISSING
-        setattr(cls, attr, field)
-
-    return cls
+environ = Environ(resolver)
 
 
 def settings(
@@ -622,7 +602,7 @@ def settings(
     frozen: bool = True,
     aliases: Mapping = None,
 ) -> Type[ObjectT]:
-    """Create a typed class which sets its defaults from env vars.
+    """Create a typed class which fetches its defaults from env vars.
 
     The resolution order of values is `default(s) -> env value(s) -> passed value(s)`.
 
@@ -643,31 +623,6 @@ def settings(
         `{'other_foo': 'foo'}` will locate the env var `OTHER_FOO` and place it
         on the `Bar.foo` attribute.
 
-    Notes
-    -----
-    Environment variables are resolved at compile-time, so updating your env after your
-    typed classes are loaded into the namespace will not work.
-
-    If you are using dotenv based configuration, you should read your dotenv file(s)
-    into the env *before* initializing the module where your settings are located.
-
-    A structure might look like:
-
-    ::
-
-        my-project/
-        -- env/
-        ..  -- .env.default
-        ..  -- .env.local
-        ..      ...
-        ..  -- __init__.py  # load your dotenv files here
-        ..  -- settings.py  # define your classes
-
-
-    This will ensure your dotenv files are loaded into the environment before the Python
-    interpreter parses & compiles your config classes, since the Python parser parses
-    the init file before parsing anything else under the directory.
-
     Examples
     --------
     >>> import os
@@ -699,6 +654,38 @@ def settings(
     return settings_wrapper(_klass) if _klass is not None else settings_wrapper
 
 
+def _resolve_from_env(
+    cls: Type[ObjectT],
+    prefix: str,
+    case_sensitive: bool,
+    aliases: Mapping[str, str],
+) -> Type[ObjectT]:
+    fields = {
+        (f"{prefix}{x}".lower() if not case_sensitive else f"{prefix}{x}"): (x, y)
+        for x, y in cached_type_hints(cls).items()
+    }
+    names = {*fields, *aliases}
+    sentinel = object()
+    for k in names:
+        name = aliases.get(k, k)
+        attr, typ = fields[name]
+        field = getattr(cls, attr, sentinel)
+        if field is sentinel:
+            field = dataclasses.field()
+        elif not isinstance(field, dataclasses.Field):
+            field = dataclasses.field(default=field)
+        if (
+            field.default != dataclasses.MISSING
+            or field.default_factory != dataclasses.MISSING
+        ):
+            continue
+        factory = environ.register(t=typ, name=name)
+        field.default_factory = functools.partial(factory, var=k, ci=not case_sensitive)
+        setattr(cls, attr, field)
+
+    return cls
+
+
 PrimitiveT = Union[Dict, List, str, int, bool]
 SchemaPrimitiveT = Dict[str, PrimitiveT]
 SchemaReturnT = Union[SchemaPrimitiveT, ObjectSchemaField]
