@@ -1,6 +1,7 @@
-#!/usr/bin/env python
-# -*- coding: UTF-8 -*-
+from __future__ import annotations
+
 import dataclasses
+from types import MappingProxyType
 from typing import (
     Type,
     ClassVar,
@@ -20,7 +21,7 @@ from typing import (
 
 from typic import gen, checks, util
 from typic.types.frozendict import freeze
-from .common import BaseConstraints, ContextT, ChecksT
+from .common import BaseConstraints, ContextT, AssertionsT
 
 if TYPE_CHECKING:  # pragma: nocover
     from typic.constraints.factory import ConstraintsT  # noqa: F401
@@ -114,33 +115,40 @@ class ArrayConstraints(BaseConstraints):
     -----
     Rather than reject arrays which are not unique, we will simply make the array unique.
     """
-    values: Optional["ConstraintsT"] = None
+    values: Optional[ConstraintsT] = None
     """The constraints for which the items in the array must adhere.
 
     This can be a single type-constraint, or a tuple of multiple constraints.
     """
 
-    def _build_validator(self, func: gen.Block) -> Tuple[ChecksT, ContextT]:
-        # No need to sanity check the config.
-        # Build the code.
-        # Only make it unique if we have to. This preserves order as well.
-        if self.unique is True and util.origin(self.type) not in {set, frozenset}:
-            func.l(f"{self.VALUE} = __unique({self.VALUE})", __unique=unique)
-        # Only get the size if we have to.
-        if {self.max_items, self.min_items} != {None, None}:
-            func.l(f"size = len({self.VALUE})")
-        # Get the validation checks and context
+    def _get_assertions(self) -> AssertionsT:
         asserts: List[str] = []
-        context: Dict[str, Any] = {}
         if self.min_items is not None:
             asserts.append(f"size >= {self.min_items}")
         if self.max_items is not None:
             asserts.append(f"size <= {self.max_items}")
+        return asserts
+
+    def _build_assertions(self, func: gen.Block, assertions: AssertionsT):
+        # Only get the size if we have to.
+        if assertions:
+            if (self.max_items, self.min_items) != (None, None):
+                func.l(f"size = len({self.VALUE})")
+            BaseConstraints._build_assertions(self, func=func, assertions=assertions)
+
+    def _build_validator(
+        self, func: gen.Block, context: ContextT, assertions: AssertionsT
+    ) -> ContextT:
+        # If we don't have a natively unique type and we're supposed to be unique, make it so.
+        if self.unique is True and util.origin(self.type) not in {set, frozenset}:
+            func.l(f"{self.VALUE} = unique({self.VALUE})", unique=unique)
+        context = BaseConstraints._build_validator(self, func, context, assertions)
         # Validate the items if necessary.
         if self.values:
             o = util.origin(self.type)
             itval = "__item_validator"
             ctx = {
+                "unique": unique,
                 itval: self.values.validate,
                 o.__name__: o,
                 "_lazy_repr": util.collectionrepr,
@@ -154,7 +162,7 @@ class ArrayConstraints(BaseConstraints):
                 f")",
                 **ctx,  # type: ignore
             )
-        return asserts, context
+        return context
 
     def for_schema(self, *, with_type: bool = False) -> dict:
         schema: Dict[str, Any] = dict(
@@ -170,7 +178,7 @@ class ArrayConstraints(BaseConstraints):
 
 @util.slotted
 @dataclasses.dataclass(frozen=True, repr=False)
-class ListContraints(ArrayConstraints):
+class ListConstraints(ArrayConstraints):
     """Specific constraints pertaining to a :py:class:`list`."""
 
     type: ClassVar[Type[list]] = list
@@ -178,10 +186,45 @@ class ListContraints(ArrayConstraints):
 
 @util.slotted
 @dataclasses.dataclass(frozen=True, repr=False)
-class TupleContraints(ArrayConstraints):
+class TupleConstraints(ArrayConstraints):
     """Specific constraints pertaining to a :py:class:`tuple`."""
 
     type: ClassVar[Type[tuple]] = tuple
+    values: Union[Optional[ConstraintsT], Sequence[ConstraintsT]] = None  # type: ignore
+
+    def _build_validator(
+        self, func: gen.Block, context: ContextT, assertions: AssertionsT
+    ) -> ContextT:
+        if isinstance(self.values, Sequence):
+            if self.unique is True:
+                func.l(
+                    f"{self.VALUE} = unique({self.VALUE}, ret_type=tuple)",
+                    unique=unique,
+                )
+            item_validators = MappingProxyType(
+                {i: c.validate for i, c in enumerate(self.values)}
+            )
+            o = util.origin(self.type)
+            itval = "__item_validators"
+            ctx = {
+                "unique": unique,
+                itval: item_validators,
+                o.__name__: o,
+                "_lazy_repr": util.collectionrepr,
+            }
+            field = f"_lazy_repr({self.FNAME}, i)"
+            func.l(
+                f"{self.VALUE} = "
+                f"{o.__name__}("
+                f"({itval}[i](x, field={field}) if i in {itval} else x "
+                f"for i, x in enumerate({self.VALUE}))"
+                f")",
+                **ctx,  # type: ignore
+            )
+            return ctx
+        return ArrayConstraints._build_validator(
+            self, func=func, context=context, assertions=assertions
+        )
 
 
 @util.slotted
