@@ -1,5 +1,5 @@
-#!/usr/bin/env python
-# -*- coding: UTF-8 -*-
+from __future__ import annotations
+
 import copy
 import dataclasses
 import datetime
@@ -28,6 +28,7 @@ from typing import (
 
 import pendulum
 
+from typic.compat import Literal
 from typic.ext.json import dumps
 from typic.serde.common import SerdeFlags
 from typic.serde.resolver import resolver
@@ -98,6 +99,10 @@ class Ref(_Serializable):
 
     ref: str
 
+    @cached_property
+    def title(self) -> str:
+        return self.ref.rsplit("/", maxsplit=1)[-1]
+
 
 class StringFormat(str, enum.Enum):
     """The official string 'formats' supported by JSON Schema.
@@ -144,7 +149,9 @@ class BaseSchemaField(_Serializable):
 
     @cached_property
     def __str(self) -> str:  # pragma: nocover
-        fields = [f"type={self.type.value!r}"]
+        fields = (
+            [f"type={self.type.value!r}"] if isinstance(self.type, SchemaType) else []
+        )
         for f in dataclasses.fields(self):
             val = getattr(self, f.name)
             if (val or val in {False, 0}) and f.repr:
@@ -318,6 +325,7 @@ class ArraySchemaField(BaseSchemaField):
 
 
 SchemaFieldT = Union[
+    BaseSchemaField,
     StrSchemaField,
     IntSchemaField,
     NumberSchemaField,
@@ -388,5 +396,6 @@ SCHEMA_FIELD_FORMATS = TypeMap(
         frozenset: ArraySchemaField(uniqueItems=True, additionalItems=False),
         dict: ObjectSchemaField(),
         type(None): NullSchemaField(),
+        Literal: BaseSchemaField(),
     }
 )
