@@ -1,8 +1,10 @@
+from __future__ import annotations
+
 import dataclasses
 import functools
 import inspect
 import warnings
-from collections.abc import Callable
+from enum import Enum
 from operator import attrgetter, methodcaller
 from typing import (
     Mapping,
@@ -11,7 +13,6 @@ from typing import (
     Optional,
     ClassVar,
     Tuple,
-    Iterable,
     cast,
     Union,
     TypeVar,
@@ -23,7 +24,6 @@ from typic import checks, constraints as constr, util, strict as st
 from typic.common import (
     EMPTY,
     ORIG_SETTER_NAME,
-    SERDE_ATTR,
     SERDE_FLAGS_ATTR,
     TYPIC_ANNOS_NAME,
     ObjectT,
@@ -31,6 +31,7 @@ from typic.common import (
     ReadOnly,
 )
 from typic.compat import ForwardRef, lru_cache
+from typic.ext import json
 from typic.strict import StrictModeT
 from .binder import Binder
 from .common import (
@@ -43,6 +44,10 @@ from .common import (
     DelayedSerdeProtocol,
     ForwardDelayedAnnotation,
     DelayedAnnotation,
+    AnnotationT,
+    DeserializerT,
+    EncoderT,
+    DecoderT,
 )
 from .des import DesFactory
 from .ser import SerFactory
@@ -60,6 +65,7 @@ class Resolver:
     )
     _DYNAMIC = SerFactory._DYNAMIC
     OPTIONALS = (None, ...)
+    LITERALS = (int, bytes, str, bool, Enum, type(None))
 
     def __init__(self):
         self.des = DesFactory(self)
@@ -180,46 +186,6 @@ class Resolver:
     def delayed(self, t: Type) -> bool:
         return getattr(t, "__delayed__", False)
 
-    @lru_cache(maxsize=None)
-    def _get_serializer_proto(self, t: Type) -> SerdeProtocol:
-        tname = util.get_name(t)
-        if hasattr(t, SERDE_ATTR):
-            return getattr(t, SERDE_ATTR)
-        for attr, caller in self._DICT_FACTORY_METHODS:
-            if hasattr(t, attr):
-
-                def serializer(
-                    val,
-                    lazy: bool = False,
-                    name: util.ReprT = None,
-                    *,
-                    __prim=self.primitive,
-                    __call=caller,
-                    __tname=tname,
-                    __repr=util.joinedrepr,
-                ):
-                    name = name or __tname
-                    return {
-                        __prim(x): __prim(y, lazy=lazy, name=__repr(name, x))
-                        for x, y in __call(val).items()
-                    }
-
-                return SerdeProtocol(
-                    self.annotation(t),  # type: ignore
-                    deserializer=None,
-                    serializer=serializer,
-                    constraints=None,
-                    validator=None,
-                )
-
-        if checks.ismappingtype(t):
-            t = Mapping[Any, Any]
-        elif checks.iscollectiontype(t) and not issubclass(t, (str, bytes, bytearray)):
-            t = Iterable[Any]
-        settings = getattr(t, SERDE_FLAGS_ATTR, None)
-        serde: SerdeProtocol = self.resolve(t, flags=settings, _des=False)
-        return serde
-
     def primitive(self, obj: Any, lazy: bool = False, name: util.ReprT = None) -> Any:
         """A method for converting an object to its primitive equivalent.
 
@@ -237,8 +203,8 @@ class Resolver:
         'foo'
         >>> typic.primitive(("foo",))  # containers are converted to lists/dicts
         ['foo']
-        >>> typic.primitive(datetime.datetime(1970, 1, 1))  # note that we assume UTC
-        '1970-01-01T00:00:00+00:00'
+        >>> typic.primitive(datetime.datetime(1970, 1, 1))
+        '1970-01-01T00:00:00'
         >>> typic.primitive(b"foo")
         'foo'
         >>> typic.primitive(ipaddress.IPv4Address("0.0.0.0"))
@@ -258,7 +224,7 @@ class Resolver:
         if checks.isenumtype(t):
             obj = obj.value
             t = obj.__class__
-        proto: SerdeProtocol = self._get_serializer_proto(t)
+        proto: SerdeProtocol = self.resolve(t)
         return proto.primitive(obj, lazy=lazy, name=name)  # type: ignore
 
     def tojson(
@@ -285,8 +251,8 @@ class Resolver:
         '"foo"'
         >>> typic.tojson(("foo",))
         '["foo"]'
-        >>> typic.tojson(datetime.datetime(1970, 1, 1))  # note that we assume UTC
-        '"1970-01-01T00:00:00+00:00"'
+        >>> typic.tojson(datetime.datetime(1970, 1, 1))
+        '"1970-01-01T00:00:00"'
         >>> typic.tojson(b"foo")
         '"foo"'
         >>> typic.tojson(ipaddress.IPv4Address("0.0.0.0"))
@@ -311,9 +277,23 @@ class Resolver:
         if checks.isenumtype(t):
             obj = obj.value
             t = obj.__class__
-        proto: SerdeProtocol = self._get_serializer_proto(t)
+        proto: SerdeProtocol = self.resolve(t)
         return proto.tojson(obj, indent=indent, ensure_ascii=ensure_ascii, **kwargs)
 
+    def decode(
+        self, annotation: Type[ObjectT], value: Any, decoder: DecoderT, **kwargs
+    ) -> ObjectT:
+        proto: SerdeProtocol = self.resolve(annotation)
+        return proto.transmute(decoder(value, **kwargs))  # type: ignore
+
+    def encode(self, obj: Any, encoder: EncoderT, **kwargs) -> bytes:
+        t = obj.__class__
+        if checks.isenumtype(t):
+            obj = obj.value
+            t = obj.__class__
+        proto: SerdeProtocol = self.resolve(t)
+        return encoder(proto.primitive(obj), **kwargs)  # type: ignore
+
     @lru_cache(maxsize=None)
     def _get_configuration(self, origin: Type, flags: "SerdeFlags") -> "SerdeConfig":
         if hasattr(origin, SERDE_FLAGS_ATTR):
@@ -387,6 +367,8 @@ class Resolver:
             fields_in=fields_in,
             fields_getters=fields_getters,
             omit_values=value_omissions,
+            encoder=flags.encoder,
+            decoder=flags.decoder,
         )
 
     def annotation(
@@ -399,7 +381,7 @@ class Resolver:
         flags: "SerdeFlags" = None,
         default: Any = EMPTY,
         namespace: Type = None,
-    ) -> Union[Annotation, DelayedAnnotation, ForwardDelayedAnnotation]:
+    ) -> AnnotationT:
         """Get a :py:class:`Annotation` for this type.
 
         Unlike a :py:class:`ResolvedAnnotation`, this does not provide access to a
@@ -430,6 +412,7 @@ class Resolver:
         )
         is_strict = is_strict or checks.isstrict(non_super) or self.STRICT
         is_static = util.origin(use) not in self._DYNAMIC
+        is_literal = checks.isliteral(use)
         # Determine whether we should use the first arg of the annotation
         while checks.should_unwrap(use) and args:
             is_optional = is_optional or checks.isoptionaltype(use)
@@ -437,6 +420,7 @@ class Resolver:
             if is_optional and len(args) > 2:
                 # We can't resolve this annotation.
                 is_static = False
+                use = Union[args[:-1]]
                 break
             # Note that we don't re-assign `orig`.
             # This is intentional.
@@ -446,12 +430,24 @@ class Resolver:
             use = non_super
             args = util.get_args(use)
             is_static = util.origin(use) not in self._DYNAMIC
+            is_literal = is_literal or checks.isliteral(use)
 
+        # Only allow legal parameters at runtime, this has implementation implications.
+        if is_literal:
+            args = util.get_args(use)
+            if any(not isinstance(a, self.LITERALS) for a in args):
+                raise TypeError(
+                    f"PEP 586: Unsupported parameters for 'Literal' type: {args}. "
+                    "See https://www.python.org/dev/peps/pep-0586/"
+                    "#legal-parameters-for-literal-at-type-check-time "
+                    "for more information."
+                )
+        # The type definition doesn't exist yet.
         if use.__class__ is ForwardRef:
-            module, localns = self.__module__, {}
+            module = self.__module__
+            # Ideally we have a namespace from a parent class/function to the field
             if namespace:
                 module = namespace.__module__
-                localns = getattr(namespace, "__dict__", {})
 
             return ForwardDelayedAnnotation(
                 ref=use,
@@ -463,9 +459,12 @@ class Resolver:
                 flags=flags,
                 default=default,
                 module=module,
-                localns=localns,
+                frame=inspect.currentframe(),
             )
+        # The type definition is recursive or within a recursive loop.
         elif use is namespace or use in self.__stack:
+            # If detected via stack, we can remove it now.
+            # Otherwise we'll cause another recursive loop.
             if use in self.__stack:
                 self.__stack.remove(use)
             return DelayedAnnotation(
@@ -478,14 +477,16 @@ class Resolver:
                 flags=flags,
                 default=default,
             )
-        self.__stack.add(use)
+        # Otherwise, add this type to the stack to prevent a recursive loop from elsewhere.
+        if not checks.isstdlibtype(use):
+            self.__stack.add(use)
         serde = (
             self._get_configuration(util.origin(use), flags)
-            if is_static
+            if is_static and not is_literal
             else SerdeConfig(flags)
         )
 
-        anno = Annotation(
+        anno: Annotation = Annotation(
             resolved=use,
             origin=orig,
             un_resolved=annotation,
@@ -498,44 +499,127 @@ class Resolver:
         anno.translator = functools.partial(self.translator.factory, anno)  # type: ignore
         return anno
 
-    @lru_cache(maxsize=None)
     def _resolve_from_annotation(
         self,
-        anno: Union[Annotation, DelayedAnnotation, ForwardDelayedAnnotation],
-        _des: bool = True,
-        _ser: bool = True,
-        _namespace: Type = None,
+        anno: AnnotationT,
+        *,
+        namespace: Type = None,
     ) -> SerdeProtocol:
         if anno in self.__cache:
             return self.__cache[anno]
         if isinstance(anno, (DelayedAnnotation, ForwardDelayedAnnotation)):
             return DelayedSerdeProtocol(anno)
 
-        # FIXME: Simulate legacy behavior. Should add runtime analysis soon (#95)
-        if anno.origin is Callable:
-            _des, _ser = False, False
         # Build the deserializer
-        deserializer, validator, constraints = None, None, None
-        if _des:
-            constraints = constr.get_constraints(
-                anno.resolved, nullable=anno.optional, cls=_namespace
-            )
-            deserializer, validator = self.des.factory(
-                anno, constraints, namespace=_namespace
-            )
+        constraints = constr.get_constraints(
+            anno.resolved, nullable=anno.optional, cls=namespace
+        )
+        deserializer, validator = self.des.factory(
+            anno, constraints, namespace=namespace
+        )
         # Build the serializer
-        serializer: Optional[SerializerT] = self.ser.factory(anno) if _ser else None
+        serializer: Optional[SerializerT] = self.ser.factory(anno)
         # Put it all together
-        proto = SerdeProtocol(
+        proto = self._build_protocol(
             annotation=anno,
+            constraints=constraints,
             deserializer=deserializer,
             serializer=serializer,
-            constraints=constraints,
             validator=validator,
         )
         self.__cache[anno] = proto
         return proto
 
+    def _build_protocol(
+        self,
+        annotation: Annotation,
+        constraints: constr.ConstraintsT,
+        *,
+        deserializer: DeserializerT = None,
+        validator: constr.ValidatorT = None,
+        serializer: SerializerT = None,
+    ) -> SerdeProtocol:
+        # Pass through if for some reason there's no coercer.
+        deserialize = deserializer or (lambda o: o)
+        # Set the validator
+        validate = validator or (lambda value, *, field=None, **kwargs: value)
+        # Set the serializer
+        serialize = serializer or (lambda o, lazy=False, name=None: o)
+
+        def tojson(
+            val: ObjectT,
+            *,
+            indent: int = 0,
+            ensure_ascii: bool = False,
+            __prim=serialize,
+            __dumps=json.dumps,
+            **kwargs,
+        ) -> str:
+            return __dumps(
+                __prim(val, lazy=True),
+                indent=indent,
+                ensure_ascii=ensure_ascii,
+                **kwargs,
+            )
+
+        tojson.__qualname__ = f"{SerdeProtocol.__name__}.{tojson.__name__}"
+        tojson.__module__ = SerdeProtocol.__module__
+
+        # Set the encoder/decoder protocols.
+        # Default to JSON for wire-format
+        encode = tojson
+        if annotation.serde.encoder:
+
+            def encode(  # type: ignore
+                val: ObjectT,
+                *,
+                __prim=serialize,
+                __encode=annotation.serde.encoder,
+                **kwargs,
+            ) -> bytes:
+                return __encode(__prim(val), **kwargs)
+
+            encode.__qualname__ = f"{SerdeProtocol.__name__}.{encode.__name__}"
+            encode.__module__ = self.__class__.__module__
+
+        # Default to JSON for wire-format
+        decode = deserialize
+        if annotation.serde.decoder:
+
+            def decode(
+                val: bytes,
+                *,
+                __trans=deserialize,
+                __decode=annotation.serde.decoder,
+                **kwargs,
+            ) -> ObjectT:
+                return __trans(__decode(val, **kwargs))
+
+            decode.__qualname__ = f"{SerdeProtocol.__name__}.{decode.__name__}"
+            decode.__module__ = SerdeProtocol.__module__
+
+        # Create the translator
+        def translate(
+            val: Any, target: Type[_T], *, __factory=annotation.translator
+        ) -> _T:
+            trans = __factory(target)
+            return trans(val)
+
+        translate.__qualname__ = f"{SerdeProtocol.__name__}.{translate.__name__}"
+        translate.__module__ = SerdeProtocol.__module__
+
+        return SerdeProtocol(
+            annotation=annotation,
+            constraints=constraints,
+            deserialize=deserialize,
+            decode=decode,
+            serialize=serialize,
+            encode=encode,  # type: ignore
+            validate=validate,
+            translate=translate,  # type: ignore
+            tojson=tojson,  # type: ignore
+        )
+
     @lru_cache(maxsize=None)
     def resolve(
         self,
@@ -547,8 +631,6 @@ class Resolver:
         is_optional: bool = None,
         is_strict: bool = None,
         namespace: Type = None,
-        _des: bool = True,
-        _ser: bool = True,
     ) -> SerdeProtocol:
         """Get a :py:class:`SerdeProtocol` from a given annotation or type.
 
@@ -584,7 +666,6 @@ class Resolver:
         --------
         :py:class:`SerdeProtocol`
         """
-        # self.__stack.clear()
         # Extract the meta-data.
         anno = self.annotation(
             annotation=annotation,
@@ -595,7 +676,7 @@ class Resolver:
             flags=flags,
             namespace=namespace,
         )
-        resolved = self._resolve_from_annotation(anno, _des, _ser, namespace)
+        resolved = self._resolve_from_annotation(anno, namespace=namespace)
         self.__stack.clear()
         return resolved
 
@@ -669,11 +750,7 @@ class Resolver:
                 param = param.replace(default=...)
 
             resolved = self.resolve(
-                annotation,
-                parameter=param,
-                name=name,
-                is_strict=strict,
-                namespace=obj,
+                annotation, name=name, parameter=param, is_strict=strict, namespace=obj
             )
             ann[name] = resolved
         try:
